import { SCORMStorageType, NormalizeReadKeyValue, NormalizeWriteKeyValue, NormalizeInteractionKeyValue, NormalizeObjectiveKeyValue, SuccessStatus, CompletionStatus, MergedStatus, FilterScormKeyValue, FilterScormPresets } from "./types";
import { LogLevel, MinLogLevel } from "./logger";
export interface SCORMInterface {
    SCORM: {
        init: () => boolean;
        quit: () => boolean;
        get: (key: string) => any;
        set: (key: string, value: any) => boolean;
        save: () => boolean;
        version: "1.2" | "2004" | "local";
    };
    UTILS: {
        trace: (msg: string, level: LogLevel) => void;
    };
    debug: {
        isActive: boolean;
    };
}
type DebounceOptions = {
    isActive: boolean;
    delayInMs: number;
};
type DebugOptions = {
    minLogLevel: MinLogLevel;
    sendLogs: boolean;
    context: {
        env?: string;
        service?: string;
    };
};
interface createWrapperInterface<CurrentStorageType extends SCORMStorageType> {
    baseStorageType: CurrentStorageType;
    scormAPIInterface?: SCORMInterface;
    shouldStoreEverythingInSuspendData?: boolean;
    debounceCommits?: Partial<DebounceOptions>;
    debug?: Partial<DebugOptions>;
    moduleId?: string;
    filteredScormValues?: FilterScormPresets | Partial<FilterScormKeyValue<CurrentStorageType>>;
}
/**
 * SCORM storage types
 * @typedef {"1.2" | "2004" | "local"} SCORMStorageType
 */
/**
 * Creates an interface between the LMS API and the custom module
 * @param {Object} config - The wrapper configuration
 * @param {SCORMStorageType} config.baseStorageType - The targeted SCORM storage type
 * @param {Object} config.scormAPIInterface - A way to override pipewek behaviour
 * @param {boolean} [config.shouldStoreEverythingInSuspendData=false] - Indicates whether objectives and interactions should be stored in suspend data
 * @param {Object} [config.debounceCommits = {}] - Object indicating how to debounce commits (if needed)
 * @param {boolean} config.debounceCommits.isActive - Activate debounce or not
 * @param {number} config.debounceCommits.delayInMs - Commit delay
 * @param {Object} [config.debug = {}] - Object indicating how to provide debug data
 * @param {MinLogLevel} config.debug.minLogLevel - Filter applied on log levels printed
 * @param {boolean} config.debug.sendLogs - Tells whether logs should be forwarded to Datadog
 * @param {Object} config.debug.context - Add some contexts to Datadog logs
 * @param {string} config.debug.context.env - Context environment (production, preprod... for instance)
 * @param {string} config.debug.context.service - Name of the service (product name for instance)
 * @returns A SCORM interface to communicate with LMS
 */
declare const create: <CurrentStorageType extends SCORMStorageType>({ baseStorageType, scormAPIInterface, shouldStoreEverythingInSuspendData, debounceCommits, debug, moduleId, filteredScormValues, }: createWrapperInterface<CurrentStorageType>) => {
    version: CurrentStorageType;
    init: ({ startSession }: {
        isDebug?: boolean;
        startSession?: boolean;
    }) => Promise<boolean>;
    close: () => Promise<boolean>;
    safelySetProgressMeasure: (newProgessMeasure: number) => Promise<void>;
    safelySetScore: (newScore: number) => Promise<void>;
    safelySetScoreScaled: (newScore: number) => Promise<void>;
    safelySetCompleteStatus: (targetedStatus: CurrentStorageType extends "1.2" ? MergedStatus : CompletionStatus) => Promise<void>;
    safelySetSuccessStatus: (targetedStatus: CurrentStorageType extends "1.2" ? MergedStatus : SuccessStatus) => Promise<void>;
    getBaseData: <K extends keyof import("./types").NormalizeRWKeyValue<StorageType> | keyof import("./types").NormalizeROKeyValue<StorageType>>(key: K) => Promise<NormalizeReadKeyValue<CurrentStorageType>[K]>;
    setBaseData: <K_1 extends keyof import("./types").NormalizeRWKeyValue<StorageType_1> | keyof import("./types").NormalizeWOKeyValue<StorageType_1>>(key: K_1, value: NormalizeWriteKeyValue<CurrentStorageType>[K_1]) => Promise<void>;
    setInteractionData: <K_2 extends keyof NormalizeInteractionKeyValue<StorageType_2>>(key: K_2, index: number, data: NormalizeInteractionKeyValue<CurrentStorageType>[K_2]) => Promise<void>;
    getInteractionData: <K_3 extends keyof NormalizeInteractionKeyValue<StorageType_3>>(key: K_3, index: number) => Promise<NormalizeInteractionKeyValue<CurrentStorageType>[K_3]>;
    setObjectiveData: <K_4 extends keyof NormalizeObjectiveKeyValue<StorageType_4>>(key: K_4, index: number, data: NormalizeObjectiveKeyValue<CurrentStorageType>[K_4]) => Promise<void>;
    getObjectiveData: <K_5 extends keyof NormalizeObjectiveKeyValue<StorageType_5>>(key: K_5, index: number) => Promise<NormalizeObjectiveKeyValue<CurrentStorageType>[K_5]>;
    getGlobalScoreRaw: () => Promise<number | null>;
    getGlobalScoreScaled: () => Promise<number | null>;
    getGlobalScoreMin: () => Promise<number | null>;
    getGlobalScoreMax: () => Promise<number | null>;
    getProgressMeasure: () => Promise<number | null>;
};
export { create as createWrapper };
